@@ -39,12 +39,12 @@ our @ISA = qw(Exporter Unicode::LineBreak);
 use Carp qw(croak carp);
 use Encode qw(is_utf8);
 use MIME::Charset;
-use Unicode::LineBreak qw(getcontext);
+use Unicode::LineBreak qw(getcontext INDIRECT);
 
 ### Globals
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-our $VERSION = '0.001_11';
+our $VERSION = '0.002';
 
 ### Public Configuration Attributes
 our $Config = {
@@ -63,19 +63,19 @@ my %FORMAT_FUNCS = (
 	my $str = shift;
 	if ($action =~ /^so[tp]/) {
 	    $self->{_} = {};
-	    $self->{_}->{MaxColumns} = $self->{MaxColumns};
-	    $self->config(MaxColumns => 0) if $str =~ /^>/;
+	    $self->{_}->{ColumnsMax} = $self->{ColumnsMax};
+	    $self->config(ColumnsMax => 0) if $str =~ /^>/;
 	} elsif ($action eq "") {
 	    $self->{_}->{line} = $str;
 	} elsif ($action eq "eol") {
 	    return $self->{Newline};
 	} elsif ($action =~ /^eo/) {
-	    if (length $self->{_}->{line} and $self->{MaxColumns}) {
+	    if (length $self->{_}->{line} and $self->{ColumnsMax}) {
 		$str = $self->{Newline}.$self->{Newline};
 	    } else {
 		$str = $self->{Newline};
 	    }
-	    $self->config(MaxColumns => $self->{_}->{MaxColumns});
+	    $self->config(ColumnsMax => $self->{_}->{ColumnsMax});
 	    delete $self->{_};
 	    return $str;
 	}
@@ -175,18 +175,26 @@ It may be string or instance of L<MIME::Charset> object.
 If a special value C<"_UNICODE_"> is specified, result will be Unicode string.
 Default is the value of Charset option.
 
+=item CharactersMax
+
+=item ColumnsMin
+
+=item ColumnsMax
+
 =item HangulAsAL
 
 =item LegacyCM
 
-=item MaxColumns
-
 =item Newline
 
 =item NSKanaAsID
 
 =item SizingMethod
 
+=item UrgentBreaking
+
+=item UserBreaking
+
 See L<Unicode::LineBreak/Options>.
 
 =back
@@ -199,8 +207,9 @@ sub config {
     my $self = shift;
     my %params = @_;
     my @opts = qw{Charset Language OutputCharset};
-    my @lbopts = qw{HangulAsAL LegacyCM MaxColumns Newline NSKanaAsID
-			SizingMethod};
+    my @lbopts = qw{CharactersMax ColumnsMin ColumnsMax
+			HangulAsAL LegacyCM Newline NSKanaAsID
+			SizingMethod UrgentBreaking UserBreaking};
 
     # Get config.
     if (scalar @_ == 1) {
@@ -412,7 +421,7 @@ sub unfold {
 			$a_cls = $self->getlbclass($n);
 			$a_cls = 'AL' if $a_cls eq 'CM';
 
-			if ($self->getlbrule($b_cls, $a_cls) eq 'INDIRECT') {
+			if ($self->getlbrule($b_cls, $a_cls) == INDIRECT) {
 			    $result .= ' ';
 			}
 		    }
